import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Panel;
import java.awt.Toolkit;

import java.awt.event.MouseAdapter;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseEvent;

import luchr.lang.Collection;

class BoardPanel extends Panel
{
	static final short
		BLOCKX			 = 110,			 BLOCKY			 =	50,
		BLOCKSIZEX	 =	58,			 BLOCKSIZEY	 =	38,
		ZEROHEIGHT	 =	40,			 RECTWIDTH		= 102;
	static final int[][][] RECTNUMBERS	=
		{{Numbers.PASSENUMBERS,Numbers.MANQUENUMBERS},
		 {Numbers.PAIRNUMBERS,Numbers.IMPAIRNUMBERS},
		 {Numbers.BLACKNUMBERS,Numbers.REDNUMBERS}};

	Image			 boardImage;
	int[]			 numbersToMark	= null;

	class LabelPainter
	{
		Font				labelFont;
		FontMetrics labelFontMetrics;
		int				 height;
		int				 ascent;

		public LabelPainter ()
		{
			super();
			labelFont = new Font("SansSerif",Font.PLAIN,10);
			labelFontMetrics = getToolkit().getFontMetrics(labelFont);
			height = labelFontMetrics.getHeight();
			ascent = labelFontMetrics.getAscent();
		}

		public void drawNumber (Graphics g, int x, int y, int no)
		{
			String st = String.valueOf(no);
			int width=labelFontMetrics.stringWidth(st)+4;

			int startx = x-width/2;
			int starty = y-height/2;

			g.setColor(Color.black);
			g.drawRect(startx,starty,width,height);
			g.setColor(Color.lightGray);
			g.fillRect(startx+1,starty+1,width-1,height-1);

			g.setColor(Color.magenta);
			g.setFont(labelFont);
			g.drawString(st,startx+2,starty+ascent);
		}

		public final void drawEinsatz (final Graphics g)
		{
			Collection eColl = ProgManager.getEinsatzColl();
			eColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
				{
					NumbersChosen nc = (NumbersChosen)(item);
					int x = BLOCKX+((nc.numbers[0]+2)%3)*BLOCKSIZEX;
					int y = BLOCKY+((nc.numbers[0]-1)/3)*BLOCKSIZEY;
					switch (nc.markType)
					{
						case nc.MT_ONE:
							if (nc.numbers[0]!=0)
							{x+=BLOCKSIZEX/2;y+=BLOCKSIZEY/2;} else
							{x=(int)(BLOCKX+1.5*BLOCKSIZEX);
							 y=(int)(BLOCKY-0.5*ZEROHEIGHT);}
							break;

						case nc.MT_TWOH: x+=BLOCKSIZEX;y+=BLOCKSIZEY/2;break;
						case nc.MT_TWOV: x+=BLOCKSIZEX/2;y+=BLOCKSIZEY;break;

						case nc.MT_FOUR: x+=BLOCKSIZEX;y+=BLOCKSIZEY;break;

						case nc.MT_THREEL: x+=3*BLOCKSIZEX;
						case nc.MT_THREER: y+=BLOCKSIZEY/2;break;

						case nc.MT_FOURZL: x=BLOCKX+3*BLOCKSIZEX;y=BLOCKY;break;
						case nc.MT_FOURZR: x=BLOCKX;y=BLOCKY;break;

						case nc.MT_SIXL: x+=3*BLOCKSIZEX;
						case nc.MT_SIXR: y+=BLOCKSIZEY;break;

						case nc.MT_TWELVER: x+=BLOCKSIZEX/2;
							y=(int)(BLOCKY+12.5*BLOCKSIZEY);break;
						case nc.MT_TWELVEDL:
							x=(int)(BLOCKX-(2.5-nc.position)*RECTWIDTH/3);
							y=(int)(BLOCKY+12.5*BLOCKSIZEY);
							break;
						case nc.MT_TWELVEDR:
							x=(int)(BLOCKX+3*BLOCKSIZEX+(nc.position+0.5)*RECTWIDTH/3);
							y=(int)(BLOCKY+12.5*BLOCKSIZEY);
							break;

						case nc.MT_EIGHTEEN:
							if (nc.position<10)
							{
								x=BLOCKX-RECTWIDTH/2;
								y=(int)(BLOCKY+(nc.position+0.5)*4*BLOCKSIZEY);
							} else
							{
								x=BLOCKX+3*BLOCKSIZEX+RECTWIDTH/2;
								y=(int)(BLOCKY+(nc.position-9.5)*4*BLOCKSIZEY);
							}
							break;
					}
					drawNumber(g,x,y,nc.einsatz);
				}
			});
		}
	}

	LabelPainter labelPainter;

	// ---- DoubleBuffering
	Dimension backbufsize;
	Image		 backbufim;
	Graphics	backbufgra=null;

	public BoardPanel ()
	{
		super();

		addMouseMotionListener(new MouseMotionAdapter () {
			public void mouseMoved (MouseEvent e)
			{
				setMarkedNumbers(getNumbersChosenByPoint(e.getX(),e.getY()).numbers);
				repaint();
			}
		});

		addMouseListener(new MouseAdapter () {
			public void mousePressed (MouseEvent e)
			{
				if ((e.getModifiers() & e.BUTTON1_MASK)!=0)
				{
					if (e.isControlDown())
					{
						ProgManager.subWindowManager.showProbability
							(getNumbersChosenByPoint(e.getX(),e.getY()));
						return;
					}
					if (ProgManager.getCredit()<=0) return;
					NumbersChosen nc = getNumbersChosenByPoint(e.getX(),e.getY());
					if (nc.markType==nc.MT_NO) return;
					EinsatzDialog ed =
						new EinsatzDialog(ProgManager.subWindowManager.getDialogFrame(),
						"Einsatz",1,ProgManager.getCredit());
					if (ed.endAction!=ed.OK) return;
					nc.einsatz=ed.betrag;
					ProgManager.addEinsatz(nc);
				}
				if ((e.getModifiers() & e.BUTTON3_MASK)!=0)
					ProgManager.deleteEinsatz(getNumbersChosenByPoint(e.getX(),e.getY()));
			}
		});
	}

	public void initData ()
	{
		boardImage = Toolkit.getDefaultToolkit().getImage("Board.Gif");
		labelPainter = new LabelPainter();
	}


	public final void update (Graphics g)
	{
		paint(g);
	}
	public final void paint (Graphics g)
	{
		Dimension size = getSize();
		if (backbufgra==null || !size.equals(backbufsize))
		{
			backbufsize=size;
			backbufim=createImage(size.width,size.height);
			backbufgra=backbufim.getGraphics();
		}

		if (backbufgra.drawImage(boardImage,0,0,getBackground(),this))
		{
			if (numbersToMark!=null) drawMarkedNumbers(backbufgra);
			labelPainter.drawEinsatz(backbufgra);
		}

		g.drawImage(backbufim,0,0,this);
	}

	public Dimension getPreferredSize ()
	{
		return new Dimension(400,560);
	}

	public final void setMarkedNumbers (int[] numbersToMark)
	{
		this.numbersToMark = numbersToMark; 
	}
	public final void drawMarkedNumbers (Graphics g)
	{
		g.setColor(Color.white);
		for (int i=0; i<numbersToMark.length; i++)
		if (numbersToMark[i]==0)
			g.drawRect(BLOCKX+5,BLOCKY-ZEROHEIGHT+5,3*BLOCKSIZEX-10,BLOCKSIZEY-10);
			else
			g.drawRect(BLOCKX+((numbersToMark[i]-1)%3)*BLOCKSIZEX+5,
								 BLOCKY+((numbersToMark[i]-1)/3)*BLOCKSIZEY+5,
								 BLOCKSIZEX-10,BLOCKSIZEY-10);
	}

	public final NumbersChosen getNumbersChosenByPoint(final int iX, final int iY)
	{
		final short pX=(short)iX,pY=(short)iY;
		if ((pX>=BLOCKX-2) && (pX<=BLOCKX+3*BLOCKSIZEX+3) &&
				(pY>=BLOCKY-2) && (pY<=BLOCKY+12*BLOCKSIZEY+3))
		{
			byte start=(byte)((pY-BLOCKY)/BLOCKSIZEY-1);
			byte ende=(byte)(start+2);
			if (start<0) start=0;
			if (ende>12) ende=12;
			for (byte z1=start; z1<ende; z1++)
				for (byte z2=0; z2<3; z2++)
				{
					final short no=(short)(z1*3+z2+1);
					final short x = (short)(BLOCKX+z2*BLOCKSIZEX);
					final short y = (short)(BLOCKY+z1*BLOCKSIZEY);
					// --- Check: One Number	(1-36)
					if ((pX>=x+2) && (pX<=x+BLOCKSIZEX-3) &&
							(pY>=y+2) && (pY<=y+BLOCKSIZEY-3))
					{
						int[] temp = {no};
						return new NumbersChosen(NumbersChosen.MT_ONE,temp);
					} else
					// --- Check: Two Numbers (Horizontal)
					if ((no%3!=0) &&
							(pX>=x+BLOCKSIZEX-2) && (pX<=x+BLOCKSIZEX+1) &&
							(pY>=y+3) && (pY<=y+BLOCKSIZEY-4))
					{
						int[] temp = {no,no+1};
						return new NumbersChosen(NumbersChosen.MT_TWOH,temp);
					}	else
					// --- Check: Two Numbers (Vertical)
					if ((no<34) &&
							(pX>=x+3) && (pX<=x+BLOCKSIZEX-4) &&
							(pY>=y+BLOCKSIZEY-2) && (pY<=y+BLOCKSIZEY+1))
					{
						int[] temp = {no,no+3};
						return new NumbersChosen(NumbersChosen.MT_TWOV,temp);
					} else
					// --- Check: Four Numbers (without Zero)
					if ((no<34) && (no%3!=0) &&
							(pX>=x+BLOCKSIZEX-3) && (pX<=x+BLOCKSIZEX+2) &&
							(pY>=y+BLOCKSIZEY-3) && (pY<=y+BLOCKSIZEY+2))
					{
						int[] temp = {no,no+1,no+3,no+4};
						return new NumbersChosen(NumbersChosen.MT_FOUR,temp);
					} else
					// --- Check: Three Numbers (Dir Right)
					if (((no+2)%3==0) &&
							 (pX>=x-2) && (pX<=x+1) &&
							 (pY>=y+3) && (pY<=y+BLOCKSIZEY-4))
					{
						int[] temp = {no,no+1,no+2};
						return new NumbersChosen(NumbersChosen.MT_THREER,temp);
					} else
					// --- Check: Three Numbers (Dir Left)
					if ((no%3==0) &&
							(pX>=x+BLOCKSIZEX-2) && (pX<=x+BLOCKSIZEX+1) &&
							(pY>=y+3) && (pY<=y+BLOCKSIZEY-4))
					{
						int[] temp = {no-2,no-1,no};
						return new NumbersChosen(NumbersChosen.MT_THREEL,temp);
					} else
					// --- Check: Six Numbers (Dir Right)
					if (((no+2)%3==0) && (no<34) &&
							(pX>=x-2) && (pX<=x+1) &&
							(pY>=y+BLOCKSIZEY-3) && (pY<=y+BLOCKSIZEY+2))
					{
						int[] temp = {no,no+1,no+2,no+3,no+4,no+5};
						return new NumbersChosen(NumbersChosen.MT_SIXR,temp);
					} else
					// --- Check: Six Numbers (Dir Left)
					if ((no%3==0) && (no<34) &&
							(pX>=x+BLOCKSIZEX-3) && (pX<=x+BLOCKSIZEX+1) &&
							(pY>=y+BLOCKSIZEY-3) && (pY<=y+BLOCKSIZEY+2))
					{
						int[] temp = {no-2,no-1,no,no+1,no+2,no+3};
						return new NumbersChosen(NumbersChosen.MT_SIXL,temp);
					}
				}
		}
		if (pY<=BLOCKY)
		{
			// --- Check: Zero
			if ((pX>=BLOCKX+2) && (pX<=BLOCKX+3*BLOCKSIZEX-3) &&
					(pY>=BLOCKY-ZEROHEIGHT+2) && (pY<=BLOCKY-3))
			{
				int[] temp = {0};
				return new NumbersChosen(NumbersChosen.MT_ONE,temp);
			}
			// --- Check: Three Numbers plus Zero (Dir Right)
			if ((pX>=BLOCKX-2) && (pX<=BLOCKX+2) &&
					(pY>=BLOCKY-3) && (pY<=BLOCKY+2))
			{
				int[] temp = {0,1,2,3};
				return new NumbersChosen(NumbersChosen.MT_FOURZR,temp);
			}
			// --- Check: Three Numbers plus Zero (Dir Left)
			if ((pX>=BLOCKX+3*BLOCKSIZEX-3) && (pX<=BLOCKX+3*BLOCKSIZEX+1) &&
					(pY>=BLOCKY-3) && (pY<=BLOCKY+2))
			{
				int[] temp = {0,1,2,3};
				return new NumbersChosen(NumbersChosen.MT_FOURZL,temp);
			}
		}
		if (pY>=BLOCKY+12*BLOCKSIZEY+2)
		{
			// --- Check: Twelve Numbers in a row
			for (int i=0; i<3; i++)
				if ((pX>=BLOCKX+i*BLOCKSIZEX+2) &&
						(pX<=BLOCKX+(i+1)*BLOCKSIZEX-3) &&
						(pY<=BLOCKY+13*BLOCKSIZEY-3))
				{
					int[] temp = {1+i,4+i,7+i,10+i,13+i,16+i,19+i,
												22+i,25+i,28+i,31+i,34+i};
					return new NumbersChosen(NumbersChosen.MT_TWELVER,temp);
				}
			// --- Check: Twelve Numbers in a block (left)
			for (int i=0; i<3; i++)
				if ((pX>=BLOCKX-RECTWIDTH+i*RECTWIDTH/3+2) &&
						(pX<=BLOCKX-RECTWIDTH+(i+1)*RECTWIDTH/3-3) &&
						(pY<=BLOCKY+13*BLOCKSIZEY-3))
				{
					int[] temp = new int[12];
					for (int j=0; j<12; j++) temp[j]=12*i+j+1;
					return new NumbersChosen(NumbersChosen.MT_TWELVEDL,i,temp);
				}
			// --- Check: Twelve Numbers in a block (right)
			for (int i=0; i<3; i++)
				if ((pX>=BLOCKX+3*BLOCKSIZEX+i*RECTWIDTH/3+2) &&
						(pX<=BLOCKX+3*BLOCKSIZEX+(i+1)*RECTWIDTH/3-3) &&
						(pY<=BLOCKY+13*BLOCKSIZEY-3))
				{
					int[] temp = new int[12];
					for (int j=25,k=0; j<37; j++,k++) temp[k]=j-12*i;
					return new NumbersChosen(NumbersChosen.MT_TWELVEDR,i,temp);
				}
		}
		// --- Check: Eighteen Numbers
		for (int z1=0; z1<3; z1++)
			for (int z2=0; z2<2; z2++)
				if ((((z2==0) && (pX>=BLOCKX-RECTWIDTH+2) &&
								(pX<=BLOCKX-3)) ||
						 ((z2==1) && (pX>=BLOCKX+3*BLOCKSIZEX+2) &&
								(pX<=BLOCKX+3*BLOCKSIZEX+RECTWIDTH-3))) &&
						(pY>=BLOCKY+z1*4*BLOCKSIZEY+2) &&
						(pY<=BLOCKY+(z1+1)*4*BLOCKSIZEY-3))
			 {
				 int[] temp=new int[18];
				 System.arraycopy(RECTNUMBERS[z1][z2],0,temp,0,18);
				 return new NumbersChosen(NumbersChosen.MT_EIGHTEEN,z2*10+z1,temp);
			 }
		return new NumbersChosen(NumbersChosen.MT_NO,null);
	}
}
