import java.awt.Font;
import java.awt.Frame;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import java.io.File;

import luchr.io.IniFile;

import luchr.lang.Collection;

public class ProgManager 
{
	private static int							 credit;
	private static EinsatzCollection einsatzColl;

	public	static SubWindowManager	subWindowManager;
	public	static IniFileManager		iniFileManager;

	class IniFileManager
	{
		private IniFile.Manager iniman;

		public IniFileManager () {super();}

		public void initIniFile ()
		{
			File f = new File("Roulette.Ini");
			iniman = new IniFile.Manager(f);
			try
			{
				if (f.exists()) iniman.readFile();
			} catch (java.io.IOException ex)
				{}
		}

		public void doneIniFile ()
		{
			try
			{
				iniman.writeFile();
			} catch (java.io.IOException ex)
				{
					new ErrorDialog(subWindowManager.getDialogFrame(),"Datei-Fehler",
						"Kann nicht in Ini-Datei schreiben!",
						subWindowManager.getErrorFont());
				}
		}
	}

	class SubWindowManager implements SubWindowSupervisor
	{
		private final String[]	subWindowName =
			{"Roulette-Tisch","Kapital","Zahl","Protokoll",
			 "Gewinnuebersicht","Haeufigkeit"};
		private final String[]	subWindowSection =
			{"BoardWindow","CreditWindow","NumberWindow","ProtocolWindow",
			 "ProfitWindow","ProbabilityWindow"};

		private final String			mainWindowName		= "Roulette";
		private final String			mainWindowSection = "MainWindow";

		private MainWindow				mainWindow;
		private BoardWindow			 boardWindow;			// (0)
		private CreditWindow			creditWindow;		 // (1)
		private NumberWindow			numberWindow;		 // (2)
		private ProtocolWindow		protocolWindow;	 // (3)
		private ProfitWindow			profitWindow;		 // (4)
		private ProbabilityWindow probabilityWindow; // (5)
		private SubWindow[]			 subWindow	 = new SubWindow[6];

		private Font						errorFont;

		public SubWindowManager () {super();}

		public void subWindowClosing (String title)
		{
			for (int i=0; i<subWindow.length; i++)
				if (subWindow[i].getTitle().equals(title))
				{
					subWindow[i].setVisible(!subWindow[i].isVisible());
					mainWindow.checkbox[i].setState(subWindow[i].isVisible());
				}
		}

		public int getSubWindowCount ()
		{
			return subWindow.length;
		}

		public String getSubWindowTitle (int index)
		{
			return subWindowName[index];
		}

		public void initWindows ()
		{
			errorFont = new Font("Serif",Font.BOLD,20);
			mainWindow = new MainWindow(getIniSection(mainWindowSection),
				mainWindowName,this);

			subWindow[0]=boardWindow = new BoardWindow(
				getIniSection(subWindowSection[0]),subWindowName[0],this);

			subWindow[1]=creditWindow = new CreditWindow(
				getIniSection(subWindowSection[1]),subWindowName[1],this,credit);

			subWindow[2]=numberWindow = new NumberWindow(
				getIniSection(subWindowSection[2]),subWindowName[2],this);
			numberWindow.startButton.addActionListener(new ActionListener () {
				public void actionPerformed (ActionEvent e)
				{
					checkAndPayProfit();
					probabilityWindow.updateProbability(numberWindow.erg);
				}
			});

			subWindow[3]=protocolWindow = new ProtocolWindow(
				getIniSection(subWindowSection[3]),subWindowName[3],this);

			subWindow[4]=profitWindow = new ProfitWindow (
				getIniSection(subWindowSection[4]),subWindowName[4],this);

			subWindow[5]=probabilityWindow = new ProbabilityWindow (
				getIniSection(subWindowSection[5]),subWindowName[5],this);

			updateMainCheckboxes();
		}

		public void doneWindows ()
		{
			for (int i=0; i<subWindow.length; i++) subWindow[i].done();
			mainWindow.done();
		}

		private IniFile.Section getIniSection (String sectionName)
		{
			if (!iniFileManager.iniman.hasSection(sectionName))
				iniFileManager.iniman.insertSection(new IniFile.Section(sectionName));
			return iniFileManager.iniman.getSection(sectionName);
		}

		public void updateMainCheckboxes ()
		{
			for (int i=0; i<subWindow.length; i++)
				mainWindow.checkbox[i].setState(subWindow[i].isVisible());
		}

		public Frame getDialogFrame () {return mainWindow;}
		public Font	getErrorFont () {return errorFont;}

		public void repaintBoardPanel ()
		{
			boardWindow.boardPanel.repaint(); 
		}

		public void updateProtocol (EinsatzCollection coll, boolean showprofit)
		{
			protocolWindow.updateProtocol(coll,showprofit);
		}

		public void updateProfits (EinsatzCollection coll)
		{
			profitWindow.updateProfits(coll);
		}

		public void setCredit (int credit)
		{
			creditWindow.setCredit(credit);
		}

		public void showProbability (NumbersChosen nc)
		{
			if (nc.markType==nc.MT_NO) return;
			double erg=0;
			for (int i=0; i<nc.numbers.length; i++)
				erg+=probabilityWindow.getNumberProbability(nc.numbers[i]);
			if (erg>=0 && erg<=100)
				new ErrorDialog(mainWindow,"Haeufigkeit",
					"erwartet: "+100.0*nc.numbers.length/36.0d+"\n"+
					"tatsaechlich: "+String.valueOf(erg),errorFont);
		}
	}

	private ProgManager () {super();}
	public static void startProgram ()
	{
		ProgManager progManager = new ProgManager ();
		(iniFileManager=progManager.new IniFileManager()).initIniFile();
		initVars();
		(subWindowManager=progManager.new SubWindowManager()).initWindows();
	}
	public static void initVars ()
	{
		credit = 0;
		einsatzColl = new EinsatzCollection(5,5);
	}

	public static void endProgram ()
	{
		subWindowManager.doneWindows();
		iniFileManager.doneIniFile();
		System.exit(0);
	}

	public static void newGame ()
	{
		EinsatzDialog ed = new EinsatzDialog(
			subWindowManager.getDialogFrame(),"Anfangskapital",1,1000);
		if (ed.endAction!=ed.OK) return;
		credit=ed.betrag;
		einsatzColl.deleteAll();
		einsatzChanged();
		creditChanged();
	}

	private static void einsatzChanged ()
	{
		subWindowManager.repaintBoardPanel();
		subWindowManager.updateProtocol(einsatzColl,false);
		subWindowManager.updateProfits(einsatzColl);
	}
	private static void einsatzFinished ()
	{
		subWindowManager.repaintBoardPanel();
		subWindowManager.updateProtocol(einsatzColl,true);
		einsatzColl.deleteAll();
	}
	public static Collection getEinsatzColl () {return einsatzColl;}
	private static NumbersChosen getEinsatz (final NumbersChosen obj)
	{
		return (NumbersChosen)(
			einsatzColl.firstThat(new Collection.Iteration () {
				public boolean doComparison (Object item)
				{
					return obj.equalsLocation((NumbersChosen)(item));
				}
			}));
	}
	public static void addEinsatz (NumbersChosen obj)
	{
		NumbersChosen nc = getEinsatz(obj);
		if (nc!=null) nc.addEinsatz(obj.einsatz); else
			try
			{
				einsatzColl.insert(obj);
			} catch (Collection.ObjectAlreadyInCollException e)
				{
					throw new InternalError("This cannot happen!");
				}
		addCredit(-obj.einsatz);
		einsatzChanged();
	}
	public static void deleteEinsatz (final NumbersChosen obj)
	{
		NumbersChosen nc = getEinsatz(obj);
		if (nc==null) return;
		try
		{
			addCredit(nc.einsatz);
			einsatzColl.delete(nc);
		} catch (Collection.ObjectNotFoundException e)
			{
				throw new InternalError("This cannot happen!");
			}
		einsatzChanged();
	}

	private static void creditChanged ()
	{
		subWindowManager.setCredit(credit);		
	}														 

	public static int getCredit () {return credit;}
	public static void addCredit (int betrag)
	{
		credit += betrag;
		creditChanged();
	}

	public static void checkAndPayProfit ()
	{
		einsatzColl.calcErhalten(subWindowManager.numberWindow.erg);
		credit += einsatzColl.getErhalten();
		creditChanged();
		einsatzFinished();
	}
}
