package luchr.awt;

import java.awt.Button;
import java.awt.Dialog;
import java.awt.Dimension;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.Insets;
import java.awt.Label;
import java.awt.Point;
import java.awt.TextField;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import luchr.awt.FramedPanel;

public class InputDialog extends Dialog
{
	static public final int
		ID_NOTDEFINED = -1, ID_CANCEL = 0, ID_BUTTON0 = 1, ID_BUTTON1 = 2;

	public FramedPanel textPanel,buttonPanel;
	public Label			 label;
	public TextField	 textField;
	public Button[]		button = new Button[2];
	public int answer = ID_NOTDEFINED;

	public InputDialog (Frame parent, String title,
		String labelText, String defaultString)
	{
		super(parent,title,true);

		addWindowListener(new WindowAdapter () {
			public void windowClosing (WindowEvent e)
				{endDialog(ID_CANCEL);}
		});

		setLayout(new GridLayout(2,1,0,5));

		// ---- textPanel
		{
			textPanel = new FramedPanel(FramedPanel.DRAW_UP,
				new GridBagLayout());

			GridBagConstraints sco = new GridBagConstraints();
			// ---- Label
			{
				label=new Label(labelText);
				sco.gridx=0;sco.gridy=0;
				sco.gridwidth=1;sco.gridheight=1;
				sco.fill=sco.NONE;
				sco.insets=new Insets(1,0,0,0);
				sco.anchor=sco.CENTER;
				sco.weightx=0;sco.weighty=0;
				textPanel.add(label,sco);
			}
			// ---- TextField
			{
				textField=new TextField(defaultString);
				textField.selectAll();
				textField.addKeyListener(new KeyAdapter () {
					public void keyPressed (KeyEvent e)
						{if (e.getKeyCode()==e.VK_ENTER) endDialog(ID_BUTTON0);}
				});
				sco.gridx=1;sco.gridy=0;
				sco.gridwidth=1;sco.gridheight=1;
				sco.fill=sco.HORIZONTAL;
				sco.insets=new Insets(0,0,0,0);
				sco.anchor=sco.CENTER;
				sco.weightx=1;sco.weighty=0;
				textPanel.add(textField,sco);
			}
			add(textPanel);
		}
		// ---- buttonPanel
		{
			buttonPanel = new FramedPanel(FramedPanel.DRAW_NOTHING,
				new GridLayout(1,2,15,0));
			// ---- Button (0)
			{
				button[0]=new Button("Ok");
				button[0].addActionListener(new ActionListener () {
					public void actionPerformed (ActionEvent e)
						{endDialog(ID_BUTTON0);}
				});
				buttonPanel.add(button[0]);
			}
			// ---- Button (1)
			{
				button[1]=new Button("Cancel");
				button[1].addActionListener(new ActionListener () {
					public void actionPerformed (ActionEvent e)
						{endDialog(ID_BUTTON1);}
				});
				buttonPanel.add(button[1]);
			}
			add(buttonPanel);
		}
	}

	public InputDialog (Frame parent, String title, String labelText)
	{
		this(parent,title,labelText,"");
	}

	public int execute ()
	{
		pack();
		Dimension dScreen = getToolkit().getScreenSize();
		Dimension dOwn = getSize();
		setLocation(new Point(
			(dScreen.width-dOwn.width)/2,(dScreen.height-dOwn.height)/2));
		setResizable(true);
		show();
		return answer;
	}

	protected void endDialog (int reason)
	{
		answer=reason;
		dispose();
	}
}
