package luchr.awt;

import java.awt.GridLayout;
import java.awt.Panel;
import java.awt.Scrollbar;
import java.awt.TextField;

import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.TextEvent;
import java.awt.event.TextListener;

public class IntegerEdit extends Panel
{
	protected TextField textfield;
	protected Scrollbar scrollbar;

	public IntegerEdit (int preferredValue, int minimumValue,
		int maximumValue)
	{
		super();

		if (minimumValue>maximumValue)
		{
			int temp=minimumValue;
			minimumValue=maximumValue;
			maximumValue=temp;
		}

		if (preferredValue<minimumValue) preferredValue=minimumValue;
		if (preferredValue>maximumValue) preferredValue=maximumValue;

		final int 
			prefValue=preferredValue,
			minValue=minimumValue,
			maxValue=maximumValue;

		setLayout(new GridLayout(2,1,0,0));

		textfield = new TextField(Math.max(
			String.valueOf(minValue).length(),
			String.valueOf(maxValue).length()
		));
		textfield.addKeyListener(new KeyAdapter () {
			final int[] validKeyCode =
				{8,37,39,48,49,50,51,52,53,54,55,56,57,58,
				 96,97,98,99,100,101,102,103,104,105,109,127,155,189};
			public void keyPressed (KeyEvent e)
			{
				int keycode=e.getKeyCode();
				boolean valid = false;
				for (int i=0; i<validKeyCode.length; i++)
					if (keycode==validKeyCode[i]) {valid=true;break;}
				if (!valid) e.consume();
			}
		});
		textfield.addTextListener(new TextListener () {
			public void textValueChanged (TextEvent e)
			{
				boolean changeText=false;
				try
				{
					int number;
					if (textfield.getText().equals(""))
					{
						number = prefValue;
						changeText=true;
					} else number=Integer.valueOf(textfield.getText()).intValue();
					if (number<minValue) {number=minValue;changeText=true;}
					if (number>maxValue) {number=maxValue;changeText=true;}
					scrollbar.setValue(number);
					if (changeText) textfield.setText(String.valueOf(number));
				} catch (NumberFormatException ex)
					{
						scrollbar.setValue(prefValue);
						textfield.setText(String.valueOf(prefValue));
					}
			}
		});
		textfield.setText(String.valueOf(prefValue));
		add(textfield);

		scrollbar = new Scrollbar(Scrollbar.HORIZONTAL,prefValue,
			1,minValue,maxValue+1);
		scrollbar.addAdjustmentListener(new AdjustmentListener () {
			public void adjustmentValueChanged (AdjustmentEvent e)
			{
				scrollbar.setValue(e.getValue());
				textfield.setText(String.valueOf(e.getValue()));
			}
		});
		add(scrollbar);
	}

	public int getValue ()
	{
		return scrollbar.getValue();
	}

	public void setValue (int value)
	{
		textfield.setText(String.valueOf(value));
	}
}
