package luchr.io;

import luchr.math.Interval;
import luchr.math.IntervalCollection;

import luchr.lang.Collection;

import luchr.io.IniFile.Entry;
import luchr.io.IniFile.Section;
import luchr.io.IniFile.Manager;

import java.io.File;
import java.io.FileWriter;

import java.util.Enumeration;

public class IniLoader
{
	static public abstract class EntryChecker extends Entry
		implements Entry.Controller
	{
		protected Entry	 client										= null;
		protected boolean isLoadFinished						= false;
		protected boolean keepControllerAfterLoad	 = false;
		protected boolean allowControllerChange		 = true;
		protected boolean insIfNotInIni						 = true;

		public EntryChecker (String key) {super(key);}

		public String toString () {return "IniLoader.EntryChecker: "+getKey();}

		/* getInsIfNotInIni */
		public boolean getInsIfNotInIni () {return insIfNotInIni;}

		/* getKeepControllerAfterLoad */
		public boolean getKeepControllerAfterLoad ()
			{return keepControllerAfterLoad;}

		/* getAllowControllerChange */
		public boolean getAllowControllerChange () {return allowControllerChange;}

		/* getIsLoadFinished */
		public boolean getIsLoadFinished () {return isLoadFinished;}

		/* debug */
		public void debug ()
		{
			super.debug();
			System.out.println(toString());
			System.out.println("isLoadFinished: "+isLoadFinished);
			System.out.println("keepControllerAfterLoad: "+keepControllerAfterLoad);
			if (keepControllerAfterLoad)
				System.out.println("allowControllerChange: "+allowControllerChange);
			System.out.println("insIfNotInIni: "+insIfNotInIni);
		}

		/* setKey */
		public void setKey (String key)
		{
			super.setKey(key);
			if (client!=null) client.setKey(key);
		}

		/* setInsIfNotInIni */
		public void setInsIfNotInIni (boolean b)
			{if (!isLoadFinished) insIfNotInIni=b;}

		/* setKeepControllerAfterLoad */
		public void setKeepControllerAfterLoad (boolean b)
			{if (!isLoadFinished) keepControllerAfterLoad=b;}

		/* setAllowControllerChange */
		public void setAllowControllerChange (boolean b)
			{if (!isLoadFinished) allowControllerChange=b;}

		/* acceptClient */
		public boolean acceptClient (Entry e, boolean inInit)
		{
			if (client!=null)
				if (client.getOwner()!=null) return false;
			if (!inInit && !e.getKey().equalsIgnoreCase(getKey())) return false;

			cutOffClient();
			client=e;
			return true;
		}

		/* setupClient */
		public void setupClient (Entry e, boolean inInit) {;}

		/* clientOwnerHasChanged */
		public void clientOwnerHasChanged (Section o, Section n)
			{if (n==null) cutOffClient();}

		/* cutOffClient */
		protected void cutOffClient ()
		{
			if (client==null) return;
			client.setController(null);
		}

		/* relaseClient */
		public void releaseClient (Entry e) {client=null;}

		/* canClientChangeController */
		public boolean canClientChangeController (Entry.Controller o,
																							Entry.Controller n)
		{
			if (!isLoadFinished) return true;
			return allowControllerChange;
		}

		/* canClientChangeKey */
		public boolean canClientChangeKey (Entry e, String newKey)
		{
			boolean erg=newKey.equalsIgnoreCase(getKey());
			if (erg) return true;
			if (!client.getKey().equalsIgnoreCase(getKey()))
				client.setKey(getKey());
			return false;
		}

		/* informLoadFinished */
		protected void informLoadFinished ()
		{
			if (!getKeepControllerAfterLoad() && client!=null)
				client.setController(null);
			isLoadFinished=true;
		}

		/* writeEntry */
		public void writeEntry (final BufferedWriter bw) throws
			java.io.IOException
		{
			bw.writeln("			e.setKeepControllerAfterLoad("+
				getKeepControllerAfterLoad()+");");
			bw.writeln("			e.setAllowControllerChange("+
				getAllowControllerChange()+");");
			bw.writeln("			e.setInsIfNotInIni("+
				getInsIfNotInIni()+");");
		}

		/* isValueValid */
		abstract public boolean isValueValid (String testvalue);

		/* getDefaultValue */
		abstract public String getDefaultValue ();

		/* canClientChangeValue */
		abstract public boolean canClientChangeValue (Entry e, String newValue);
	}

	static public class StringEntryChecker extends EntryChecker
	{
		protected String		 defaultString;
		protected Collection validStringColl;

		public StringEntryChecker (String key, String defaultString,
			int capacity, int delta)
		{
			super(key);
			validStringColl=new Collection(capacity,delta);
			setDefaultValue(defaultString);
		}
		public StringEntryChecker (String key, String defaultString,
			int capacity) {this(key,defaultString,capacity,5);}
		public StringEntryChecker (String key, String defaultString)
			{this(key,defaultString,0);}
		public StringEntryChecker (String key) {this(key,"");}

		public String toString ()
			{return "IniLoader.StringEntryChecker: "+getKey();}

		/* debug */
		public void debug ()
		{
			super.debug();
			System.out.println("defaultString: "+defaultString);
			validStringColl.debug();
		}

		/* getValidCollectionClone */
		public Collection getValidCollectionClone ()
			{return (Collection)(validStringColl.clone());}

		/* getDefaultValue */
		public String getDefaultValue () {return getDefaultString();}

		/* getDefaultString */
		public String getDefaultString () {return defaultString;}

		/* setDefaultValue */
		public void setDefaultValue (String st)
		{
			if (st==null) st="";
			defaultString=st;
			addValidString(defaultString);
		}

		/* findString */
		protected String findString (final String st)
		{
			if (st==null) return null;
			return (String)(
			validStringColl.firstThat(new Collection.Iteration () {
				public boolean doComparison (Object item)
					{return st.equals(item);}
			}));
		}

		/* addValidString */
		public void addValidString (String st)
		{
			if (st==null || findString(st)!=null) return;
			try
			{
				validStringColl.insert(st);
			} catch (Collection.ObjectAlreadyInCollException ex)
				{ // this cannot happen
					ex.printStackTrace();
				}
		}

		/* deleteValidString */
		public void deleteValidString (String st)
		{
			String toDelete=findString(st);
			if (toDelete==null	|| toDelete.equals(defaultString)) return;
			try
			{
				validStringColl.delete(toDelete);
			} catch (Collection.ObjectNotFoundException ex)
				{ // this cannot happen
					ex.printStackTrace();
				}
		}

		/* deleteAllValidString */
		public void deleteAllValidStrings ()
		{
			validStringColl.deleteAll();
			addValidString(defaultString);
		}

		/* canClientChangeValue */															
		public boolean canClientChangeValue (Entry e, String newValue)
			{return isValueValid(newValue);}

		/* isValueValid */
		public boolean isValueValid (String testValue)
			{return findString(testValue)!=null;}

		/* writeEntry */
		public void writeEntry (final BufferedWriter bw) throws
			java.io.IOException
		{
			validStringColl.trimToSize();
			bw.writeln("		// ---- StringEntry: "+getKey());
			bw.writeln("		{");
			bw.writeln("			StringEntryChecker e =");
			bw.writeln("				new StringEntryChecker(\""+getKey()+"\",\""+
				getDefaultString()+"\", "+validStringColl.getCapacity()+", "+
				validStringColl.getDelta()+");");
			super.writeEntry(bw);
			Enumeration enum = validStringColl.items();
			while (enum.hasMoreElements())
				bw.writeln("			e.addValidString(\""+
					enum.nextElement().toString()+"\");");
			bw.writeln("			secChecker.insertEntry(e);");
			bw.writeln("		}");
		}
	}

	static public class BooleanEntryChecker extends EntryChecker
	{
		boolean defaultBoolean;

		public BooleanEntryChecker (String key, boolean defaultBoolean)
		{
			super(key);
			setDefaultValue(defaultBoolean);
		}
		public BooleanEntryChecker (String key) {this(key,false);}

		public String toString ()
			{return "IniLoader.BooleanEntryChecker: "+getKey();}

		/* debug */
		public void debug ()
		{
			super.debug();
			System.out.println("defaultBoolean: "+defaultBoolean);
		}

		/* getDefaultValue */
		public String getDefaultValue ()
			{return String.valueOf(getDefaultBoolean());}

		/* getDefaultBoolean */
		public boolean getDefaultBoolean () {return defaultBoolean;}

		/* setDefaultValue */
		public void setDefaultValue (boolean b) {defaultBoolean=b;}

		/* canClientChangeValue */
		public boolean canClientChangeValue (Entry e, String newValue)
			{return isValueValid(newValue);}

		/* isValueValid */
		public boolean isValueValid (String newValue)
		{
			if (newValue==null) return false;
			return (newValue.equalsIgnoreCase("TRUE") ||
							newValue.equalsIgnoreCase("FALSE"));
		}

		/* writeEntry */
		public void writeEntry (final BufferedWriter bw) throws
			java.io.IOException
		{
			bw.writeln("		// ---- BooleanEntry: "+getKey());
			bw.writeln("		{");
			bw.writeln("			BooleanEntryChecker e =");
			bw.writeln("				new BooleanEntryChecker(\""+getKey()+"\","+
				getDefaultValue()+");");
			super.writeEntry(bw);
			bw.writeln("			secChecker.insertEntry(e);");
			bw.writeln("		}");
		}
	}

	static public class EmptyEntryChecker extends EntryChecker
	{
		String defaultString;

		public EmptyEntryChecker (String key, String defaultString)
		{
			super(key);
			setDefaultValue(defaultString);
		}

		public String toString ()
			{return "IniLoader.EmptyEntryChecker: "+getKey();}

		/* debug */
		public void debug ()
		{
			super.debug();
			System.out.println("defaultString: "+defaultString);
		}

		/* getDefaultValue */
		public String getDefaultValue () {return getDefaultString();}

		/* getDefaultString */
		public String getDefaultString () {return defaultString;}

		/* setDefaultValue */
		public void setDefaultValue (String s) {defaultString=s;}

		/* isValueValid */
		public boolean isValueValid (String newValue)
		{
			if (newValue==null) return false;
			return !newValue.trim().equals("");
		}

		/* canClientChangeValue */
		public boolean canClientChangeValue (Entry e, String newValue)
			{return isValueValid(newValue);}

		/* writeEntry */
		public void writeEntry (final BufferedWriter bw) throws
			java.io.IOException
		{
			bw.writeln("		// ---- EmptyEntry: "+getKey());
			bw.writeln("		{");
			bw.writeln("			EmptyEntryChecker e =");
			bw.writeln("				new EmptyEntryChecker(\""+getKey()+"\",\""+
				getDefaultValue()+"\");");
			super.writeEntry(bw);
			bw.writeln("			secChecker.insertEntry(e);");
			bw.writeln("		}");
		}
	}

	static public class NumberEntryChecker extends EntryChecker
	{
		Number						 defaultNumber;
		IntervalCollection validIntervalColl;

		public NumberEntryChecker (String key, Number defaultNumber,
															 int capacity, int delta)
		{
			super(key);
			this.defaultNumber=defaultNumber;
			Interval i = new Interval(false,defaultNumber,defaultNumber,false);
			validIntervalColl = new IntervalCollection(i.getTypeIndex(),
														capacity,delta);
			addValidInterval(i);
		}
		public NumberEntryChecker (String key, Number defaultNumber,
															 int capacity)
			{this(key,defaultNumber,capacity,0);}
		public NumberEntryChecker (String key, Number defaultNumber)
			{this(key,defaultNumber,0);}

		public String toString ()
			{return "IniLoader.NumberEntryChecker: "+getKey();}

		/* debug */
		public void debug ()
		{
			super.debug();
			System.out.println("defaultNumber: "+defaultNumber.toString());
		}

		/* getValidCollectionClone */
		public IntervalCollection getValidCollectionClone ()
			{return (IntervalCollection)(validIntervalColl.clone());}

		/* getDefaultNumber */
		public Number getDefaultNumber () {return defaultNumber;}

		/* getDefaultValue */
		public String getDefaultValue () {return getDefaultNumber().toString();}

		/* setDefaultValue */
		public void setDefaultValue (Number n)
		{
			if (n==null) return;
			addValidInterval(new Interval(false,n,n,false));
			defaultNumber=n;
		}
		public void setDefaultValue (String s)
			{setDefaultValue(convertToNumber(s));}

		/* convertToNumber */
		public Number convertToNumber (String s)
		{
			if (s==null) return null;
			Number n=null;
			try
			{
				switch (getTypeIndex())
				{
					case 0: if (s.equalsIgnoreCase("min"))
										n=new Byte(Byte.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Byte(Byte.MAX_VALUE); else
									n=new Byte(s);break;
					case 1: if (s.equalsIgnoreCase("min"))
										n=new Short(Short.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Short(Short.MAX_VALUE); else
									n=new Short(s);break;
					case 2: if (s.equalsIgnoreCase("min"))
										n=new Integer(Integer.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Integer(Integer.MAX_VALUE); else
									n=new Integer(s);break;
					case 3: if (s.equalsIgnoreCase("min"))
										n=new Long(Long.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Long(Long.MAX_VALUE); else
									n=new Long(s);break;
					case 4: if (s.equalsIgnoreCase("min"))
										n=new Float(Float.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Float(Float.MAX_VALUE); else
									if (s.equalsIgnoreCase("-infinity"))
										n=new Float(Float.NEGATIVE_INFINITY);else
									if (s.equalsIgnoreCase("infinity") ||
											s.equalsIgnoreCase("+infinity"))
										n=new Float(Float.POSITIVE_INFINITY);else
									n=new Float(s);break;
					case 5: if (s.equalsIgnoreCase("min"))
										n=new Double(Double.MIN_VALUE); else
									if (s.equalsIgnoreCase("max"))
										n=new Double(Double.MAX_VALUE); else
									if (s.equalsIgnoreCase("-infinity"))
										n=new Double(Double.NEGATIVE_INFINITY);else
									if (s.equalsIgnoreCase("infinity") ||
											s.equalsIgnoreCase("+infinity"))
										n=new Double(Double.POSITIVE_INFINITY);else
									n=new Double(s);break;
					default: return null;
				}
			} catch (NumberFormatException ex) {return null;}
			return n;
		}

		/* convertToString */
		public String convertToString (Number n)
		{
			if (n==null) return null;
			switch (getTypeIndex())
			{
				case 0: if (n.byteValue()==Byte.MIN_VALUE)
									return "Byte.MIN_VALUE"; else
								if (n.byteValue()==Byte.MAX_VALUE)
									return "Byte.MAX_VALUE"; else
								return "(byte)("+n.toString()+")";
				case 1: if (n.shortValue()==Short.MIN_VALUE)
									return "Short.MIN_VALUE"; else
								if (n.shortValue()==Short.MAX_VALUE)
									return "Short.MAX_VALUE"; else
								return "(short)("+n.toString()+")";
				case 2: if (n.intValue()==Integer.MIN_VALUE)
									return "Integer.MIN_VALUE"; else
								if (n.intValue()==Integer.MAX_VALUE)
									return "Integer.MAX_VALUE"; else
								return "(int)("+n.toString()+")";
				case 3: if (n.longValue()==Long.MIN_VALUE)
									return "Long.MIN_VALUE"; else
								if (n.longValue()==Long.MAX_VALUE)
									return "Long.MAX_VALUE"; else
								return "(long)("+n.toString()+"L)";
				case 4: if (n.floatValue()==Float.MIN_VALUE)
									return "Float.MIN_VALUE"; else
								if (n.floatValue()==Float.MAX_VALUE)
									return "Float.MAX_VALUE"; else
								if (n.floatValue()==Float.POSITIVE_INFINITY)
									return "Float.POSITIVE_INFINITY"; else
								if (n.floatValue()==Float.NEGATIVE_INFINITY)
									return "Float.NEGATIVE_INFINITY"; else
								return "(float)("+n.toString()+"F)";
				case 5: if (n.doubleValue()==Double.MIN_VALUE)
									return "Double.MIN_VALUE"; else
								if (n.doubleValue()==Double.MAX_VALUE)
									return "Double.MAX_VALUE"; else
								if (n.doubleValue()==Double.POSITIVE_INFINITY)
									return "Double.POSITIVE_INFINITY"; else
								if (n.doubleValue()==Double.NEGATIVE_INFINITY)
									return "Double.NEGATIVE_INFINITY"; else
								return "(double)("+n.toString()+"D)";
			}
			return null;
		}

		/* getTypeIndex */
		public int getTypeIndex () {return validIntervalColl.getTypeIndex();}

		/* findInterval */
		protected Interval findInterval (final Interval i)
		{
			if (i==null) return null;
			return (Interval)(
			validIntervalColl.firstThat (new Collection.Iteration () {
				public boolean doComparison (Object item)
					{return i.equals(item);}
			}));
		}

		/* addValidInterval */
		public void addValidInterval (Interval i)
		{
			if (i==null || findInterval(i)!=null) return;
			try
			{
				validIntervalColl.insert(i);
			} catch (Collection.ObjectAlreadyInCollException ex)
				{ // this cannot happen
					ex.printStackTrace();
				}
		}

		/* deleteValidInterval */
		public void deleteValidInterval (Interval i)
		{
			Interval toDelete=findInterval(i);
			if (toDelete==null) return;
			try
			{
				validIntervalColl.delete(toDelete);
			} catch (Collection.ObjectNotFoundException ex)
				{ // this cannot happen
					ex.printStackTrace();
				}
			if (!validIntervalColl.isNumberInCollection(defaultNumber))
				addValidInterval(
					new Interval(false,defaultNumber,defaultNumber,false));
		}

		/* deleteAllValidIntervals */
		public void deleteAllValidIntervals ()
		{
			validIntervalColl.deleteAll();
			addValidInterval(
				new Interval(false,defaultNumber,defaultNumber,false));
		}

		/* isValueValid */
		public boolean isValueValid (String testValue)
		{
			Number n=null;
			try
			{
				switch (validIntervalColl.getTypeIndex())
				{
					case 0: n=new Byte(testValue);break;
					case 1: n=new Short(testValue);break;
					case 2: n=new Integer(testValue);break;
					case 3: n=new Long(testValue);break;
					case 4: n=new Float(testValue);break;
					case 5: n=new Double(testValue);break;
					default: throw new NumberFormatException();
				}
			} catch (NumberFormatException ex) {return false;}
			return validIntervalColl.isNumberInCollection(n); 
		}

		/* canClientChangeValue */
		public boolean canClientChangeValue (Entry e, String newValue)
			{return isValueValid(newValue);}

		/* writeEntry */
		public void writeEntry (final BufferedWriter bw) throws
			java.io.IOException
		{
			bw.writeln("		// ---- NumberEntry: "+getKey());
			bw.writeln("		{");

			String[] nType = {"Byte","Short","Integer","Long","Float","Double"};

			bw.writeln("			NumberEntryChecker e =");
			bw.writeln("				new NumberEntryChecker(\""+getKey()+"\",new "+
				nType[validIntervalColl.getTypeIndex()]+"("+
				convertToString(getDefaultNumber())+"),"+
				validIntervalColl.getCapacity()+","+validIntervalColl.getDelta()+
				");");
			super.writeEntry(bw);
			Enumeration enum=validIntervalColl.items();
			while (enum.hasMoreElements())
			{
				Interval i=(Interval)(enum.nextElement());
				bw.writeln("			e.addValidInterval(new Interval("+
					i.isMinLimitOpen()+",new "+
					nType[i.getTypeIndex()]+"("+convertToString(i.getMinLimit())+
					"),new "+
					nType[i.getTypeIndex()]+"("+convertToString(i.getMaxLimit())+
					"),"+i.isMaxLimitOpen()+"));");
			}
			bw.writeln("			secChecker.insertEntry(e);");
			bw.writeln("		}");
		}
	}

	static public class SectionChecker extends Section
		implements Section.Controller
	{
		protected Section client											= null;
		protected boolean isLoadFinished							= false;
		protected boolean keepControllerAfterLoad		 = false;
		protected boolean allowControllerChange			 = true;
		protected boolean insIfNotInIni							 = true;
		protected boolean insEntryIfNotInMaster			 = true;

		public SectionChecker (String name, int capacity, int delta)
			{super(name,capacity,delta);}
		public SectionChecker (String name, int capacity)
			{this(name,capacity,3);}
		public SectionChecker (String name) {this(name,0);}

		public String toString ()
			{return "IniLoader.SectionChecker: "+getName();}

		/* getIsLoadFinished */
		public boolean getIsLoadFinished () {return isLoadFinished;}

		/* getKeepControllerAfterLoad */
		public boolean getKeepControllerAfterLoad ()
			{return keepControllerAfterLoad;}

		/* getAllowControllerChange */
		public boolean getAllowControllerChange ()
			{return allowControllerChange;}

		/* getInsIfNotInIni */
		public boolean getInsIfNotInIni () {return insIfNotInIni;}

		/* getInsEntryIfNotInMaster */
		public boolean getInsEntryIfNotInMaster ()
			{return insEntryIfNotInMaster;}

		/* debug */
		public void debug ()
		{
			System.out.println(toString());
			System.out.println("Controller: "+
				(getController()==null?"null":getController().toString()));
			System.out.println("isLoadFinished: "+isLoadFinished);
			System.out.println("keepControllerAfterLoad: "+keepControllerAfterLoad);
			if (keepControllerAfterLoad)
				System.out.println("allowControllerChange: "+allowControllerChange);
			System.out.println("insIfNotInIni: "+insIfNotInIni);
			System.out.println("insEntryIfNotInMaster: "+insEntryIfNotInMaster);
			entryColl.debug();
		}

		/* setName */
		public void setName (String name)
		{
			super.setName(name);
			if (client!=null) client.setName(name);
		}

		/* informLoadFinished */
		protected void informLoadFinished ()
		{
			if (!getKeepControllerAfterLoad() && client!=null)
				client.setController(null);
			isLoadFinished=true;
			entryColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
				{
					((EntryChecker)(item)).informLoadFinished();
				}
			});
		}

		/* insertEntry */
		public void insertEntry (EntryChecker e) throws
			Section.InsertEntryException
			{super.insertEntry(e);}

		/* setKeepControllerAfterLoad */
		public void setKeepControllerAfterLoad (boolean b)
			{if (!isLoadFinished) keepControllerAfterLoad=b;}

		/* setAllowControllerChange */
		public void setAllowControllerChange (boolean b)
			{if (!isLoadFinished) allowControllerChange=b;}

		/* setInsIfNotInIni */
		public void setInsIfNotInIni (boolean b)
			{if (!isLoadFinished) insIfNotInIni=b;}

		/* setInsEntryIfNotInMaster */
		public void setInsEntryIfNotInMaster (boolean b)
			{if (!isLoadFinished) insEntryIfNotInMaster=b;}

		/* acceptClient */
		public boolean acceptClient (Section s, boolean inInit)
		{
			if (client!=null)
				if (client.getOwner()!=null) return false;
			if (!inInit && s.getName().equalsIgnoreCase(getName())) return false;

			cutOffClient();
			client=s;
			return true;
		}

		/* setupClient */
		public void setupClient (Section s, boolean inInit) {;}

		/* clientOwnerHasChanged */
		public void clientOwnerHasChanged (Manager o, Manager n)
			{if (n==null) cutOffClient();}

		/* cutOffClient */
		public void cutOffClient ()
		{
			entryColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
					{((EntryChecker)(item)).cutOffClient();}
			});
			if (client==null) return;
			client.setController(null);
		}

		/* releaseClient */
		public void releaseClient (Section s) {client=null;}

		/* canClientInsertEntry */
		public boolean canClientInsertEntry (Section s, Entry e)
		{
			EntryChecker c=(EntryChecker)(getEntryByKey(e.getKey()));
			if (c!=null) return true;
			return insEntryIfNotInMaster;
		}

		/* canClientDeleteEntry */
		public boolean canClientDeleteEntry (Section s, Entry e)
		{
			if (!isLoadFinished) return true;
			EntryChecker c=(EntryChecker)(getEntryByKey(e.getKey()));
			if (c==null) return true;
			return !c.getInsIfNotInIni();
		}

		/* canClientChangeName */
		public boolean canClientChangeName (Section s, String newName)
		{
			boolean erg = newName.equalsIgnoreCase(getName());
			if (erg) return true;
			if (!client.getName().equalsIgnoreCase(getName()))
				client.setName(getName());
			return false;
		}

		/* canClientChangeController */
		public boolean canClientChangeController (Section.Controller o,
																							Section.Controller n)
		{
			if (!isLoadFinished) return true;
			return allowControllerChange;
		}

		/* writeSection */
		public void writeSection (final BufferedWriter bw) throws
			java.io.IOException
		{
			entryColl.trimToSize();
			bw.writeln("	// ---- SectionChecker: "+getName());
			bw.writeln("	{");
			bw.writeln("		SectionChecker secChecker = ");
			bw.writeln("			new SectionChecker(\""+getName()+"\","+
				entryColl.getCapacity()+","+entryColl.getDelta()+");");
			bw.writeln("		secChecker.setKeepControllerAfterLoad("+
				getKeepControllerAfterLoad()+");");
			bw.writeln("		secChecker.setAllowControllerChange("+
				getAllowControllerChange()+");");
			bw.writeln("		secChecker.setInsIfNotInIni("+
				getInsIfNotInIni()+");");
			bw.writeln("		secChecker.setInsEntryIfNotInMaster("+
				getInsEntryIfNotInMaster()+");");
			Enumeration enum = entryColl.items();
			while (enum.hasMoreElements())
				((EntryChecker)(enum.nextElement())).writeEntry(bw);
			bw.writeln("		mcl.insertSection(secChecker);");
			bw.writeln("	}");
		}
	}

	static public class ManagerCheckerAndLoader extends Manager
		implements Manager.Controller,
							 SectionTuner,EntryTuner
	{
		protected Manager client									 = null;
		protected boolean isLoadFinished					 = false;
		protected boolean keepControllerAfterLoad	= false;
		protected boolean allowControllerChange		= true;
		protected boolean insSectionIfNotInMaster	= true;
		protected String	fileName								 = null;
		protected String	directoryName						= null;
		
		public ManagerCheckerAndLoader (File iniFile, int capacity, int delta)
			{super(iniFile,capacity,delta);}
		public ManagerCheckerAndLoader (File iniFile, int capacity)
			{this(iniFile,capacity,5);}
		public ManagerCheckerAndLoader (File iniFile) {this(iniFile,5);}

		public String toString ()
			{return "IniLoader.ManagerCheckerAndLoader: "+getFile().toString();}

		/* getIsLoadFinished */
		public boolean getIsLoadFinished () {return isLoadFinished;}

		/* getKeepControllerAfterLoad */
		public boolean getKeepControllerAfterLoad ()
			{return keepControllerAfterLoad;}

		/* getAllowControllerChange */
		public boolean getAllowControllerChange ()
			{return allowControllerChange;}

		/* getInsSectionIfNotInMaster */
		public boolean getInsSectionIfNotInMaster ()
			{return insSectionIfNotInMaster;}

		/* getFileName */
		public String getFileName () {return fileName;}

		/* getDirectoryName */
		public String getDirectoryName () {return directoryName;}

		/* debug */
		public void debug ()
		{
			System.out.println(toString());
			System.out.println("Controller: "+
				(getController()==null?"null":getController().toString()));
			System.out.println("isLoadFinished: "+isLoadFinished);
			System.out.println("keepControllerAfterLoad: "+keepControllerAfterLoad);
			if (keepControllerAfterLoad)
				System.out.println("allowControllerChange: "+allowControllerChange);
			System.out.println("insSectionIfNotInMaster: "+insSectionIfNotInMaster);
			secColl.debug();
		}

		/* setFile */
		public void setFile (File iniFile)
		{
			super.setFile(iniFile);
			if (client!=null) client.setFile(iniFile);
		}

		/* setFileName */
		public void setFileName (String fileName) {this.fileName=fileName;}

		/* setDirectoryName */
		public void setDirectoryName (String dirName)
			{this.directoryName=dirName;}

		/* informLoadFinished */
		protected void informLoadFinished ()
		{
			if (!getKeepControllerAfterLoad() && client!=null)
				client.setController(null);
			isLoadFinished=true;
			secColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
					{((SectionChecker)(item)).informLoadFinished();}
			});
		}

		/* setKeepControllerAfterLoad */
		public void setKeepControllerAfterLoad (boolean b)
			{if (!isLoadFinished) keepControllerAfterLoad=b;}

		/* setAllowControllerChange */
		public void setAllowControllerChange (boolean b)
			{if (!isLoadFinished) allowControllerChange=b;}

		/* setInsSectionIfNotInMaster */
		public void setInsSectionIfNotInMaster (boolean b)
			{if (!isLoadFinished) insSectionIfNotInMaster=b;}

		/* insertSection */
		public void insertSection (SectionChecker s) throws
			Manager.InsertSectionException
			{super.insertSection(s);}

		/* acceptClient */
		public boolean acceptClient (Manager m, boolean inInit)
		{
			if (client!=null) return false;
			if (!inInit && !client.getFile().equals(getFile())) return false;
			client=m;return true;
		}

		/* setupClient */
		public void setupClient (Manager m, boolean inInit) {;}

		/* releaseClient */
		public void releaseClient (Manager m) {client=null;}

		/* canClientInsertSection */
		public boolean canClientInsertSection (Manager m, Section s)
		{
			if (getSection(s.getName())!=null) return true;
			return insSectionIfNotInMaster;
		}

		/* canClientDeleteSection */
		public boolean canClientDeleteSection (Manager m, Section s)
		{
			if (!isLoadFinished) return true;
			SectionChecker sc=(SectionChecker)(getSection(s.getName()));
			if (sc==null) return true;
			return !sc.getInsIfNotInIni();
		}

		/* canClientChangeFile */
		public boolean canClientChangeFile (Manager m, File newFile)
		{
			boolean erg = getFile().equals(newFile);
			if (erg) return true;
			if (!client.getFile().equals(getFile()))
				client.setFile(getFile());
			return false;
		}

		/* canClientChangeController */
		public boolean canClientChangeController (Manager.Controller o,
																							Manager.Controller n)
		{
			if (!isLoadFinished) return true;
			return allowControllerChange;
		}

		/* getTunedSection */
		public Section getTunedSection (String name)
			{return new Section((SectionChecker)(getSection(name)),name,5,0);}

		/* getTunedEntry */
		public Entry getTunedEntry (Section section,
			String key, String value)
		{
			SectionChecker sc = (SectionChecker)(getSection(section.getName()));
			if (sc==null) return new Entry(key,value);

			EntryChecker ec = (EntryChecker)(sc.getEntryByKey(key));
			if (ec==null) return new Entry(key,value);

			String tunedValue;
			if (ec.isValueValid(value)) tunedValue=value; else
				tunedValue=ec.getDefaultValue();
			return new Entry(key,tunedValue,ec);
		}

		/* startTunedLoading */
		public Manager startTunedLoading (Manager manager)
		{
			if (manager==null)
				manager = new TunedManager(this,this,this,getFile());
			try {manager.readFile();}
				catch (java.io.IOException ex) {;}
			addMissingSections(manager);
			addMissingEntries(manager);
			informLoadFinished();
			return manager;
		}
		public Manager startTunedLoading () {return startTunedLoading(client);}

		/* addMissingSections */
		protected void addMissingSections (final Manager m)
		{
			secColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
				{
					SectionChecker sc=(SectionChecker)(item);
					if (sc.getInsIfNotInIni() && !m.hasSection(sc.getName()))
						m.insertSection(getTunedSection(sc.getName()));		
				}
			});
		}

		/* addMissingEntries */
		protected void addMissingEntries (final Manager m)
		{
			secColl.forEach(new Collection.Iteration () {
				public void doAction (Object item)
				{
					final SectionChecker sc=(SectionChecker)(item);
					final Section s=m.getSection(sc.getName());
					if (s==null) return;
					sc.entryColl.forEach(new Collection.Iteration () {
						public void doAction (Object item)
						{
							EntryChecker ec=(EntryChecker)(item);
							if (ec.getInsIfNotInIni() && !s.hasEntry(ec.getKey()))
								s.insertEntry(getTunedEntry(s,ec.getKey(),ec.getDefaultValue()));
						}
					});
				}
			});
		}

		/* writeFile */
		public void writeFile () throws
			java.io.IOException
		{
			BufferedWriter bw=null;
			try
			{
				if (directoryName==null) directoryName="";
				if (fileName==null)
				{
					fileName=getFile().toString();
					int pos=fileName.lastIndexOf('.');
					fileName=fileName.substring(0,pos)+fileName.substring(pos+1);
				} else
				{
					int pos=fileName.lastIndexOf('.');
					if (pos!=-1)
						fileName=fileName.substring(0,pos);
				}
				
				bw = new BufferedWriter(new
					FileWriter(new File(directoryName+fileName+".java")));
				bw.writeln("// \""+fileName+"\"");bw.newLine();
				bw.writeln("import java.io.File;");
				bw.writeln("import java.io.ObjectOutputStream;");
				bw.writeln("import java.io.FileOutputStream;");
				bw.newLine();
				bw.writeln("import luchr.io.IniLoader.EntryChecker;");
				bw.writeln("import luchr.io.IniLoader.BooleanEntryChecker;");
				bw.writeln("import luchr.io.IniLoader.StringEntryChecker;");
				bw.writeln("import luchr.io.IniLoader.NumberEntryChecker;");
				bw.writeln("import luchr.io.IniLoader.EmptyEntryChecker;");
				bw.writeln("import luchr.io.IniLoader.SectionChecker;");
				bw.writeln("import luchr.io.IniLoader.ManagerCheckerAndLoader;");
				bw.newLine();
				bw.writeln("import luchr.math.Interval;");
				bw.newLine();
				bw.writeln("class "+fileName+" {");
				bw.writeln("public static void main (String[] args) "+
					"throws java.io.IOException {");
				bw.writeln("	ManagerCheckerAndLoader mcl = ");
				bw.writeln("		new ManagerCheckerAndLoader(new File(\""+
												getFile().toString()+"\"),"+secColl.getCapacity()+","+
												secColl.getDelta()+");");
				bw.writeln("	mcl.setKeepControllerAfterLoad("+
					getKeepControllerAfterLoad()+");");
				bw.writeln("	mcl.setAllowControllerChange("+
					getAllowControllerChange()+");");
				bw.writeln("	mcl.setInsSectionIfNotInMaster("+
					getInsSectionIfNotInMaster()+");");
				Enumeration enum = secColl.items();
				while (enum.hasMoreElements())
					((SectionChecker)(enum.nextElement())).writeSection(bw);
				bw.writeln("ObjectOutputStream oos = new ObjectOutputStream(");
				directoryName=directoryName.replace('\\','/');
				bw.writeln("	new FileOutputStream(\""+
					directoryName+fileName+".Master"+"\"));");
				bw.writeln("oos.writeObject(mcl);");
				bw.writeln("oos.flush();oos.close();");
				bw.writeln("}");
				bw.writeln("}");
			} finally {if (bw!=null) {bw.flush();bw.close();}}
		}

		// These methods are useless for a ManagerChecker
		public Section getNewSection (String name)
			{throw new InternalError("Not supported!");}
		public Entry getNewEntry (String key, String value)
			{throw new InternalError("Not supported!");}
		public void lineWithoutSection (String line)
			{throw new InternalError("Not supported!");}
		public void notValidEntry (String line)
			{throw new InternalError("Not supported!");}
		public void readFile () {throw new InternalError("Not supported!");}
	}

	static public interface SectionTuner
	{
		public Section getTunedSection (String name);
	}

	static public interface EntryTuner
	{
		public Entry getTunedEntry (Section section,
			String key, String value);
	}

	static public class TunedManager extends Manager
	{
		protected SectionTuner sectionTuner;
		protected EntryTuner	 entryTuner;

		public TunedManager (SectionTuner sectionTuner, EntryTuner entryTuner,
												 Manager.Controller controller, File iniFile,
												 int capacity, int delta)
		{
			super(iniFile,capacity,delta,controller);
			this.sectionTuner=sectionTuner;
			this.entryTuner=entryTuner;
		}
		public TunedManager (SectionTuner sectionTuner, EntryTuner entryTuner,
												 Manager.Controller controller, File iniFile,
												 int capacity)
			{this(sectionTuner,entryTuner,controller,iniFile,capacity,5);}
		public TunedManager (SectionTuner sectionTuner, EntryTuner entryTuner,
												 Manager.Controller controller, File iniFile)
			{this(sectionTuner,entryTuner,controller,iniFile,5);}
		public TunedManager (SectionTuner sectionTuner, EntryTuner entryTuner,
												 File iniFile)
			{this(sectionTuner,entryTuner,null,iniFile);}
 
		public String toString ()
			{return "IniLoader.TunedManager: "+getFile().toString();}

		public void debug ()
		{
			System.out.println(toString());
			System.out.println("Controller: "+
				(getController()==null?"null":getController().toString()));
			System.out.println("entryTuner: "+entryTuner.toString());
			System.out.println("sectionTuner: "+sectionTuner.toString());
			secColl.debug();
		}

		public Section getNewSection (String name)
			{return sectionTuner.getTunedSection(name);}

		public Entry getNewEntry (Section s,
			String key, String value)
			{return entryTuner.getTunedEntry(s,key,value);}

		// ignore errors
		public void lineWithoutSection (String line) {;}
		public void notValidEntry (String line) {;}

		public boolean stopAfterInsertSectionException (Section s)
			{return false;}
		public boolean stopAfterInsertEntryException (Section s, Entry e)
			{return false;}
	}
}

