package luchr.lang;

public class Collection implements Cloneable, java.io.Serializable
{
	/*
	------ nested top-level classes

		Exception										(java.lang.Exception)							|pu
		IndexOutOfRangeException		 (Collection.Exception)						 |pu
		ObjectAlreadyInCollException (Collection.Exception)						 |pu
		ObjectNotFoundException			(Collection.Exception)						 |pu
		Iteration										(java.lang.Object)								 |pu,a
		Enumerator									 (java.lang.Object)								 |pu
																 [java.util.Enumeration]
	------ fields
		count																	 int										 |pr
		delta																	 int										 |pr
		items																	 Object[]								|pr

	------ constructors
		Collection(int capacity, int delta)														 |pu
		Collection (int capacity)																			 |pu
		Collection ()																									 |pu

	------ methods
		at (int index)													-> Object							 |pu,f,sy
					 *IndexOutOfRangeException*
		atDelete (int index)										-> void								 |pu,f,sy
					 *IndexOutOfRangeException*
		atInsert (int index, Object item)			 -> void								 |pu,f,sy
					 *IndexOutOfRangeException,
						ObjectAlreadyInCollException*
		checkMemory ()													-> void								 |pr
		clone()																 -> Object							 |pu,sy
		contains (Object item)									-> boolean							|pu,f
		copyInto (Object anArray[])						 -> void								 |pu,f,sy
		debug ()																-> void								 |pu,sy
		delete (Object item)										-> void								 |pu,f
					 *ObjectNotFoundException*
		deleteAll ()														-> void								 |pu,f,sy
		firstItem ()														-> Object							 |pu,f
		firstThat (Iteration iobj)							-> Object							 |pu,f,sy
		forEach (Iteration iobj)								-> void								 |pu,f,sy
		getCount ()														 -> int									|pu,f
		getDelta ()														 -> int									|pu,f
		indexOf (Object item)									 -> int									|pu,f
		indexOf (Object item, int index)				-> int									|pu,f,sy
		insert (Object item)										-> void								 |pu,sy
					 *ObjectAlreadyInCollException*
		isEmpty()															 -> boolean							|pu,f
		items ()																-> Enumerator					 |pu,f,sy
		lastIndexOf (Object item)							 -> int									|pu,f
		lastIndexOf (Object item, int index)		-> int									|pu,f,sy
		lastItem ()														 -> Object							 |pu,f
		lastThat (Iteration iobj)							 -> Object							 |pu,f,sy
		replace (int index, Object item)				-> void								 |pu,f,sy
					 *IndexOutOfRangeException*
		replace (Object olditem, Object newitem)-> void								 |pu,f
					 *ObjectNotFoundException*
		setCapacity (int capacity)							-> void								 |pu,f,sy
		setDelta (int delta)										-> void								 |pu,f,sy
		trimToSize ()													 -> void								 |pu,f,sy
	*/

	static public class Exception extends java.lang.Exception
	{
		public Exception ()
		{
		super();
		}

		public Exception (String message)
		{
			super(message);
		}
	}

	static public class IndexOutOfRangeException extends Exception
	{
		public IndexOutOfRangeException ()
		{
			super();
		}
		public IndexOutOfRangeException (String message)
		{
			super(message);
		}
	}

	static public class ObjectAlreadyInCollException extends Exception
	{
		public ObjectAlreadyInCollException ()
		{
			super();
		}
		public ObjectAlreadyInCollException (String message)
		{
			super(message);
		}
	}

	static public class ObjectNotFoundException extends Exception
	{
		public ObjectNotFoundException ()
		{
			super();
		}
		public ObjectNotFoundException (String message)
		{
			super(message);
		}
	}

	static public abstract class Iteration
	{
		public void doAction (Object item)
		{}
		public boolean doComparison (Object item)
		{
			return true;
		}
	}

	static public class Enumerator implements java.util.Enumeration
	{
		Collection coll;
		int count;

		Enumerator (Collection coll)
		{
			super();
			this.coll = coll;
			count = 0;
		}

		public boolean hasMoreElements ()
		{
			return count<coll.getCount();
		}

		public Object nextElement()
		{
			try
			{
				return coll.at(count++);
			} catch (IndexOutOfRangeException ex)
				{
					throw new java.util.NoSuchElementException(
						"Enumerator of Collection \""+coll.toString()+"\"");
				}
		}
	}

	private int			count=0;
	private int			delta;
	private Object[] items;

	public Collection (int capacity, int delta)
	{
		super();
		if (capacity<0) capacity=5;
		if (delta<0) delta=5;
		items = new Object[capacity];
		this.delta=delta;
	}
	public Collection (int capacity)
	{
		this(capacity,0);
	}
	public Collection ()
	{
		this(10);
	}

	public final int getCount () {return count;}
	public final boolean isEmpty() {return count==0;}
	public final int getDelta () {return delta;}
	public final int getCapacity () {return items.length;}
	public final synchronized void setDelta (int delta)
	{
		if (delta>=0) this.delta=delta;
		checkMemory();
	}

	/* clone */
	public synchronized Object clone()
	{
		try
		{
			Collection c = (Collection)super.clone();
			c.count = count;c.delta = delta;
			c.items = new Object[items.length];
			System.arraycopy(items,0,c.items,0,items.length);
			return c;
		} catch (CloneNotSupportedException ex)
			{ // this shoudn't happen, because we are Cloneable
				ex.printStackTrace();
				return null;
			}
	}

	/* debug */
	public synchronized void debug ()
	{
		System.out.println(toString());
		System.out.println("count				= "+count);
		System.out.println("delta				= "+delta);
		System.out.println("items.length = "+items.length);
		for (int i=0; i<count; i++)
			System.out.println("items["+i+"] = "+items[i].toString());
	}

	/* atInsert */
	public synchronized void atInsert (int index, Object item) throws
		IndexOutOfRangeException, ObjectAlreadyInCollException
	{
		if (item==null) throw
			new NullPointerException("Cannot insert object \"null\" in a Collection!");
		if (index<0 || index>count) throw
			new IndexOutOfRangeException("Cannot insert at postion "+index+
			 "\nCollection has only "+count+" items!");
		for (int i=0; i<count; i++)
			if (items[i]==item) throw
				new ObjectAlreadyInCollException("Object is already in Collection "+
					"at index "+i);
		if (items.length==count)
		{
			Object[] temp;
			if (delta==0)
				temp = new Object[Math.max(items.length<<1,1)]; else
				temp = new Object[items.length+delta];
			System.arraycopy(items,0,temp,0,items.length);
			items = temp;
		}
		if (index!=count)
			System.arraycopy(items,index,items,index+1,count-index);
		items[index]=item;
		count += 1;
	}

	/* insert */
	public synchronized void insert (Object item)
		throws ObjectAlreadyInCollException
	{
		try
		{
			atInsert(count,item);
		} catch (IndexOutOfRangeException ex)
			{ // this cannot happen !!!
				ex.printStackTrace();
			}
	}

	/* atDelete */
	public final synchronized void atDelete (int index)
		throws IndexOutOfRangeException
	{
		if (index<0 || index>=count) throw
			new IndexOutOfRangeException("Cannot delete item at "+
				"index "+index+"\nCollection has only "+count+" items!");
		if (index!=count-1)
			System.arraycopy(items,index+1,items,index,count-1-index);
		count -= 1;
		checkMemory();
	}

	/* delete */
	public final void delete (Object item)
		throws ObjectNotFoundException
	{
		if (item==null) throw
			new ObjectNotFoundException("Item \"null\" not found!");
		try
		{
			atDelete(indexOf(item));
		} catch (IndexOutOfRangeException ex)
			{ 
				throw new ObjectNotFoundException("Item \""+item.toString()+
					"\" not found!");
			}
	}

	/* deleteAll */
	public final synchronized void deleteAll ()
	{
		items = new Object[0];
		count = 0;
	}

	/* trimToSize */
	public final synchronized void trimToSize ()
	{
		if (items.length>count)
		{
			Object[] temp = new Object[count];
			System.arraycopy(items,0,temp,0,count);
			items = temp;
		}
	}

	/* copyInto */
	public final synchronized void copyInto (Object anArray[])
	{
		System.arraycopy(items,0,anArray,0,count);
	}

	/* items */
	public final synchronized Enumerator items ()
	{
		return new Enumerator(this);
	}

	/* checkMemory */
	private void checkMemory ()
	{
		if (delta==0)
		{
			if (items.length-count>items.length+1>>1)
			{
				Object[] temp = new Object[items.length+1>>1];
				System.arraycopy(items,0,temp,0,count);
				items = temp;
				checkMemory();
			}
		}
		else
		{
			if (items.length-count>=delta)
			{
				Object[] temp = new Object[items.length-delta];
				System.arraycopy(items,0,temp,0,count);
				items = temp;
				checkMemory();
			}
		}
	}

	/* at */
	public final synchronized Object at (int index)
		throws IndexOutOfRangeException
	{
		if (index<0 || index>=count) throw
			new IndexOutOfRangeException("Cannot return item at "+
				"index "+index+"\nCollection has only "+count+" items!");
		return items[index];
	}

	/* indexOf */
	public final synchronized int indexOf (Object item, int index)
	{
		int erg=-1;
		for (int i=index; i<count; i++) if (items[i]==item)
		{
			erg=i;break;
		}
		return erg;
	}
	public final int indexOf (Object item) 
	{
		return indexOf(item,0);
	}

	/* lastIndexOf */
	public final synchronized int lastIndexOf (Object item, int index) 
	{			
		int erg=-1;
		for (int i=index; i>=0; i--) if (items[i]==item)
		{
			erg=i;break;
		}
		return erg;
	}
	public final int lastIndexOf (Object item) 
	{
		return lastIndexOf(item,count-1);
	}

	/* firstItem, lastItem */
	public final Object firstItem () 
	{
		try
		{
			return at(0);
		} catch (IndexOutOfRangeException ex)
			{
				return null;
			}
	}
	public final Object lastItem () 
	{
		try
		{
			return at(count-1);
		} catch (IndexOutOfRangeException ex)
			{
				return null;
			}
	}

	/* contains */
	public final boolean contains (Object item)
	{
		return indexOf(item)!=-1;
	}

	/* replace */
	public final synchronized void replace (int index, Object item)
		throws IndexOutOfRangeException
	{
		if (index<0 || index>=count) throw
			new IndexOutOfRangeException("Cannot replace at postition "+index+
				"\nCollection has only "+count+" items!");
		items[index]=item;
	}
	public final void replace (Object olditem, Object newitem)
		throws ObjectNotFoundException
	{
		try
		{
			replace(indexOf(olditem),newitem);
		} catch (IndexOutOfRangeException ex)
			{ // this cannot happen !!!
				System.err.println(ex.getMessage());
				ex.printStackTrace();
			}
	}

	/* forEach, firstThat, lastThat */
	public final synchronized void forEach (Iteration iobj)
	{
		for (int i=0; i<count; i++) iobj.doAction(items[i]);
	}
	public final synchronized Object firstThat (Iteration iobj)
	{
		Object erg = null;
		for (int i=0; i<count; i++)
			if (iobj.doComparison(items[i]))
			{
				erg = items[i];
				break;
			}
		return erg;
	}
	public final synchronized Object lastThat (Iteration iobj)
	{
		Object erg = null;
		for (int i=count-1; i>=0; i--)
		if (iobj.doComparison(items[i]))
		{
			erg = items[i];
			break;
		}
		return erg;
	}

	/* setCapactity */
	public final synchronized void setCapacity (int capacity)
	{
		count = Math.min(count,capacity);
		Object[] temp = new Object[capacity];
		System.arraycopy(items,0,temp,0,count);
		items = temp;
	}
}
