package luchr.math;

public class Interval implements java.io.Serializable
{
	public static final boolean
		OPEN = true, CLOSED = false;

	protected Number minLimit, maxLimit;
	protected boolean minLimitOpen, maxLimitOpen;

	public Interval (boolean minLimitOpen, Number minLimit,
									 boolean maxLimitOpen, Number maxLimit) 
	{
		super();
		if (getTypeIndex(minLimit)!=getTypeIndex(maxLimit))
			throw new IllegalArgumentException("Limits dont have the same type!");
		this.minLimit=minLimit;this.minLimitOpen=minLimitOpen;
		this.maxLimit=maxLimit;this.maxLimitOpen=maxLimitOpen;
	}

	// Init with Object Number
	public Interval (boolean minLimitOpen, Number minLimit,
									 Number maxLimit, boolean maxLimitOpen)
		{this(minLimitOpen,minLimit,maxLimitOpen,maxLimit);}
	public Interval (Number minLimit, boolean minLimitOpen,
									 Number maxLimit, boolean maxLimitOpen)
		{this(minLimitOpen,minLimit,maxLimitOpen,maxLimit);}
	public Interval (Number minLimit, boolean minLimitOpen,
									 boolean maxLimitOpen, Number maxLimit)
		{this(minLimitOpen,minLimit,maxLimitOpen,maxLimit);}

	// Init with int
	public Interval (boolean minLimitOpen, int minLimit,
									 boolean maxLimitOpen, int maxLimit)
		{this(minLimitOpen, new Integer(minLimit), maxLimitOpen,
					new Integer(maxLimit));}
	public Interval (boolean minLimitOpen, int minLimit,
									 int maxLimit, boolean maxLimitOpen)
		{this(minLimitOpen,new Integer(minLimit),new Integer(maxLimit),
					maxLimitOpen);}
	public Interval (int minLimit, boolean minLimitOpen,
									 int maxLimit, boolean maxLimitOpen)
		{this(new Integer(minLimit),minLimitOpen,new Integer(maxLimit),
					maxLimitOpen);}
	public Interval (int minLimit, boolean minLimitOpen,
									 boolean maxLimitOpen, int maxLimit)
		{this(new Integer(minLimit),minLimitOpen,maxLimitOpen,
					new Integer(maxLimit));}

	// Init widh Double
	public Interval (boolean minLimitOpen, double minLimit,
									 boolean maxLimitOpen, double maxLimit)
		{this(minLimitOpen,new Double(minLimit),maxLimitOpen,
					new Double(maxLimit));}
	public Interval (boolean minLimitOpen, double minLimit,
									 double maxLimit, boolean maxLimitOpen)
		{this(minLimitOpen,new Double(minLimit),new Double(maxLimit),
					maxLimitOpen);}
	public Interval (double minLimit, boolean minLimitOpen,
									 double maxLimit, boolean maxLimitOpen)
		{this(new Double(minLimit),minLimitOpen,new Double(maxLimit),
					maxLimitOpen);}
	public Interval (double minLimit, boolean minLimitOpen,
									 boolean maxLimitOpen, double maxLimit)
		{this(new Double(minLimit),minLimitOpen,maxLimitOpen,
					new Double(maxLimit));}

	public String toString ()
	{
		String st="";
		switch (getTypeIndex())
		{
			case 0: st="byte";break;
			case 1: st="short";break;
			case 2: st="integer";break;
			case 3: st="long";break;
			case 4: st="float";break;
			case 5: st="double";break;
		}
		return "Interval ("+st+"): "+(isEmpty()?"empty":
			(minLimitOpen?"]":"[")+minLimit+","+maxLimit+
			(maxLimitOpen?"[":"]"));
	}

	/* equals */
	public boolean equals (Object o)
	{
		if (o==null) return false;
		if (!(o instanceof Interval)) return false;
		Interval i = (Interval)o;
		return minLimitOpen==i.minLimitOpen &&
					 maxLimitOpen==i.maxLimitOpen &&
					 minLimit.equals(i.minLimit) &&
					 maxLimit.equals(i.maxLimit);
	}

	/* setMinLimit */
	public void setMinLimit (Number minLimit, boolean minLimitOpen)
	{
		if (getTypeIndex(minLimit)!=getTypeIndex())
			throw new IllegalArgumentException("Limits dont have the same type!");
		this.minLimit=minLimit;
		this.minLimitOpen=minLimitOpen;
	}
	public void setMinLImit (boolean minLimitOpen, Number minLimit)
		{setMinLimit(minLimit,minLimitOpen);}

	/* getMinLimit */
	public Number getMinLimit () {return minLimit;}

	/* isMinLimitOpen */
	public boolean isMinLimitOpen () {return minLimitOpen;}

	/* setMaxLimit */
	public void setMaxLimit (Number maxLimit, boolean maxLimitOpen)
	{
		if (getTypeIndex(maxLimit)!=getTypeIndex())
			throw new IllegalArgumentException("Limits dont have the same type!");
		this.maxLimit=maxLimit;
		this.maxLimitOpen=maxLimitOpen;
	}
	public void setMaxLimit (boolean maxLimitOpen, Number maxLimit)
		{setMaxLimit(maxLimit,maxLimitOpen);}

	/* getMaxLimit */
	public Number getMaxLimit () {return maxLimit;}

	/* isMaxLimitOpen */
	public boolean isMaxLimitOpen () {return maxLimitOpen;}

	/* isEmpty */
	public boolean isEmpty ()
	{
		switch (getTypeIndex())
		{
			case 0: case 1: case 2:
			case 3: return isEmpty_Long();
			case 4:
			case 5: return isEmpty_Double();
		}
		return false;
	}
	protected boolean isEmpty_Long ()
	{
		if (!minLimitOpen && !maxLimitOpen)
			return minLimit.longValue()>maxLimit.longValue();
		return minLimit.longValue()>=maxLimit.longValue();
	}
	protected boolean isEmpty_Double ()
	{
		if (!minLimitOpen && !maxLimitOpen)
			return minLimit.doubleValue()>maxLimit.doubleValue();
		return minLimit.doubleValue()>=maxLimit.doubleValue();
	}

	/* getTypeIndex */
	protected int getTypeIndex (Number n)
	{
		if (n instanceof Byte) return 0;
		if (n instanceof Short) return 1;
		if (n instanceof Integer) return 2;
		if (n instanceof Long) return 3;
		if (n instanceof Float) return 4;
		if (n instanceof Double) return 5;
		throw new NumberFormatException("Cannot determine type of number!");
	}
	public int getTypeIndex () {return getTypeIndex(minLimit);}

	/* isNumberInInterval */
	public boolean isNumberInInterval (Number n)
	{
		if (isEmpty()) return false;
		int t=getTypeIndex(n);
		switch (getTypeIndex())
		{
			case 0: case 1: case 2:
			case 3: if (t<=3) return isNumberInInterval_Long(n); else
												return isNumberInInterval_Double(n);
			case 4:
			case 5: return isNumberInInterval_Double(n);
		}
		return false;
	}
	protected boolean isNumberInInterval_Long (Number n)
	{
		boolean erg=true;
		if (minLimitOpen) erg=n.longValue()>minLimit.longValue(); else
											erg=n.longValue()>=minLimit.longValue();
		if (!erg) return false;
		if (maxLimitOpen) erg=n.longValue()<maxLimit.longValue(); else
											erg=n.longValue()<=maxLimit.longValue();
		return erg;
	}
	protected boolean isNumberInInterval_Double (Number n)
	{
		boolean erg=true;
		if (minLimitOpen) erg=n.doubleValue()>minLimit.doubleValue();else
											erg=n.doubleValue()>=minLimit.doubleValue();
		if (!erg) return false;
		if (maxLimitOpen) erg=n.doubleValue()<maxLimit.doubleValue();else
											erg=n.doubleValue()<=maxLimit.doubleValue();
		return erg;
	}
}
