package luchr.awt;

import java.io.ObjectInputStream;

import java.util.StringTokenizer;

import java.awt.Canvas;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Image;
import java.awt.FontMetrics;
import java.awt.Graphics;



public class MultiLineLabel extends Canvas
{
	static final long serialVersionUID = -6996448182960524442L;

	public static final int		LEFT = 0;
	public static final int		CENTER = 1;
	public static final int		RIGHT = 2;
	// 
	protected String		 label;
	protected int				margin_width, margin_height;
	protected int				alignment;
	//
	protected String[]	 line;
	protected int				num_lines;
	//
	protected boolean		fmetrics_calculated = false;
	protected int				max_width;
	protected int				line_height,line_ascent;
	protected int[]			line_width;
	//
	private transient Dimension	backbufsize;
	private transient Image			backbufim;
	private transient Graphics	 backbufgra = null;

	public MultiLineLabel (String label, int alignment,
												 Font font,int margin_width, int margin_height)
	{
		super();
		this.label = label;
		this.alignment = alignment;
		setFont(font);
		this.margin_width = margin_width;
		this.margin_height = margin_height;
		calcLines();
	}
	public MultiLineLabel (String label, int alignment, Font font)
	{
		this(label,alignment,font,5,5);
	}
	public MultiLineLabel (String label, int alignment)
	{
		this(label,alignment,null);
	}
	public MultiLineLabel (String label)
	{
		this(label,LEFT);
	}
	public MultiLineLabel ()
	{
		this("");
	}

	private void readObject (ObjectInputStream in) throws
		java.io.IOException, ClassNotFoundException
	{
		in.defaultReadObject();
		backbufgra=null;
	}

	public String paramString ()
	{
		String str = "MultiLineLabel,align=";
		switch(alignment)
		{
			case LEFT:	 str+="left";break;
			case CENTER: str+="center";break;
			case RIGHT:	str+="right";break;
		}
		return str+",label="+label;
	}

	public String getLabel () {return label;}
	public void setLabel (String label)
	{
		this.label = label;
		calcLines();
	}

	public void setFont (Font font)
	{
		super.setFont(font);
		fmetrics_calculated = false;
	}

	public int getMarginWidth () {return margin_width;}
	public void setMarginWidth (int margin_width)
	{
		this.margin_width = margin_width;
	}

	public int getMarginHeight () {return margin_height;}
	public void setMarginHeight (int margin_height)
	{
		this.margin_height = margin_height;
	}

	public int getAlignment () {return alignment;}
	public void setAlignment (int alignment)
	{
		this.alignment = alignment;
	}

	protected void calcLines ()
	{
		StringTokenizer st = new StringTokenizer (label,"\n");
		num_lines = st.countTokens();
		line = new String[num_lines];
		line_width = new int[num_lines];
		for (int i=0; i<num_lines; i++) line[i] = st.nextToken();
		fmetrics_calculated=false;
	}

	protected void calcFontMetrics ()
	{
		FontMetrics fm = getFontMetrics(getFont());
		line_height = fm.getHeight();
		line_ascent = fm.getAscent();
		max_width = 0;
		for (int i=0; i<num_lines; i++)
		{
			line_width[i] = fm.stringWidth(line[i]);
			if (line_width[i]>max_width) max_width = line_width[i];
		}
		fmetrics_calculated = true;
	}

	public Dimension getPreferredSize ()
	{
		if (!fmetrics_calculated) calcFontMetrics();
		return new Dimension(max_width+2*margin_width,
			num_lines*line_height+2*margin_height);
	}
	public Dimension getMinimumSize ()
	{
		return getPreferredSize();
	}

	/*
	public void setBounds (int x, int y, int width, int height)
	{
		super.setBounds(x,y,width,height);
		repaint();
	}
	*/

	public void update (Graphics g)
	{
		paint(g);
	}
	public void paint (Graphics g)
	{
		Dimension size = getSize();
		if (backbufgra==null || !size.equals(backbufsize))
		{
			backbufsize = size;
			backbufim = createImage(size.width,size.height);
			backbufgra = backbufim.getGraphics();
		}
		backbufgra.setColor(getBackground());
		backbufgra.fillRect(0,0,size.width,size.height);
		backbufgra.setColor(getForeground());
		int x,y;
		if (!fmetrics_calculated) calcFontMetrics();
		y = line_ascent + (size.height-num_lines*line_height)/2;
		for (int i=0; i<num_lines; i++, y += line_height)
		{
			switch(alignment)
			{
				default:
				case LEFT:	 x = margin_width;break;
				case CENTER: x = (size.width - line_width[i])/2;break;
				case RIGHT:	x = size.width - line_width[i] - margin_width;break;
			}
			backbufgra.drawString(line[i],x,y);
		}
		g.drawImage(backbufim,0,0,this);
	}

}
